mesdata = pd.read_excel(r'C:\Users\tedch\Documents\Thesis\Code\mesdata.xls')
X=mesdata
y=mesdata['class of diagnosis']
num_feat=['age','duration of asbestos exposure', 'duration of symptoms', 'white blood', 'cell count (WBC)',
          'platelet count (PLT)', 'sedimentation', 'blood lactic dehydrogenise (LDH)', 'alkaline phosphatise (ALP)',
          'total protein', 'albumin', 'glucose', 'pleural lactic dehydrogenise', 'pleural albumin',
          'pleural glucose', 'C-reactive protein (CRP)' , 'pleural protein']

cat_feat=['gender', 'city', 'asbestos exposure', 'type of MM', 'diagnosis method',
       'keep side', 'cytology', 'dyspnoea', 'ache on chest', 'weakness',
       'habit of cigarette', 'performance status', 'hemoglobin (HGB)',
       'dead or not', 'pleural effusion', 'pleural thickness on tomography',
       'pleural level of acidity (pH)', 'class of diagnosis']

mesdata_num=mesdata[num_feat]

# transform=['duration of symptoms', 'platelet count (PLT)', 'alkaline phosphatise (ALP)', 'glucose',
#            'pleural lactic dehydrogenise', 'cell count (WBC)', 'blood lactic dehydrogenise (LDH)']
#
# X[transform]=X[transform].apply(np.log)

X_num = X[num_feat]
X_cat = X[cat_feat]

# Corr heatmap
corr = X_num.corr()
mask = np.triu(np.ones_like(corr, dtype=bool))
cmap = sns.diverging_palette(230, 20, as_cmap=True)
sns.heatmap(
    data=X_num.corr(), annot=True, mask=mask, cmap=cmap
)

# Num features summary
num_sum=X[num_feat].agg([np.mean, np.std])
from sklearn import preprocessing
mesdata_norm = preprocessing.normalize(mesdata_num)
X_norm=preprocessing.normalize(X_num)

X_norm=pd.DataFrame(X_norm, columns=X_num.columns)
mesdata_norm=pd.DataFrame(mesdata_norm, columns=mesdata_num.columns)
X_norm.hist(figsize=(20, 20))
mesdata_norm.hist(figsize=(20, 20))

plt.figure(figsize=(10,10))
plt.hist(mesdata_norm['duration of symptoms'], label='Non-transformed')
plt.hist(X_num['duration of symptoms'], label='Log-transformed')
plt.title("Duration of Symptoms Superimposed")
plt.legend(loc='upper right')

fig, ax = plt.subplots(6, 3, figsize=(20, 20))
for ax, feature in zip(ax.flat, X_num.columns):
    sns.histplot(X_num[feature], ax=ax)

# Cat features summary
for i in cat_feat:
    cat_sum=X[i].value_counts()
    print(cat_sum)

fig, ax = plt.subplots(3, 6, figsize=(20, 10))
for variable, subplot in zip(cat_feat, ax.flatten()):
    sns.countplot(mesdata[variable], ax=subplot, data=mesdata, hue='class of diagnosis')
    for label in subplot.get_xticklabels():
        label.set_rotation(90)

chisqvar = ['gender', 'type of MM', 'diagnosis method', 'keep side', 'class of diagnosis']

f, axes = plt.subplots(1,4)
sns.countplot(mesdata['gender'], data=mesdata, hue='class of diagnosis', ax=axes[0])
sns.countplot(mesdata['type of MM'], data=mesdata, hue='class of diagnosis', ax=axes[1])
sns.countplot(mesdata['diagnosis method'], data=mesdata, hue='class of diagnosis', ax=axes[2])
sns.countplot(mesdata['keep side'], data=mesdata, hue='class of diagnosis', ax=axes[3])

# Target Summary
mesdata.groupby('class of diagnosis').size().plot(kind='pie', autopct='%.2f')
sns.countplot(x="class of diagnosis", data=mesdata)

# Outliers IQR
Q1=mesdata.quantile(0.25)
Q3=mesdata.quantile(0.75)
IQR=Q3-Q1

X = mesdata[~((mesdata < (Q1 - 1.5 * IQR)) |(mesdata > (Q3 + 1.5 * IQR))).any(axis=1)]
y= X['class of diagnosis']
X=X.drop(['class of diagnosis', 'diagnosis method', 'pleural protein'],axis=1)

# Outliers Z Score
X=mesdata[(np.abs(stats.zscore(mesdata)) < 3).all(axis=1)]
y= X['class of diagnosis']
X=X.drop(['class of diagnosis', 'diagnosis method', 'pleural protein'],axis=1)

# Chi2
chi_scores = chi2(X_cat,y)
chi2_features = SelectKBest(chi2, k = 10)
X_kbest_features = chi2_features.fit_transform(X_cat, y)

p_values = pd.Series(chi_scores[1],index = X_cat.columns)
p_values.sort_values(ascending = False , inplace = True)
p_values.plot.bar()

from scipy.stats import chi2_contingency
chi2_contingency(X_cat)

# ANOVA
f_test = f_classif(X, y)
p_values = pd.Series(f_test[1],index = X.columns)
p_values.sort_values(ascending = False , inplace = True)
p_values.plot.bar()

# Mutual Info
mut = mutual_info_classif(X,y)
p_values = pd.Series(mut,index = X.columns)
p_values.sort_values(ascending = False , inplace = True)
p_values.plot.bar()

# Outliers Scatter
sns.set_theme(style="ticks")
sns.pairplot(mesdata, y_vars=X_num.iloc[:,0:6], x_vars=X_num.iloc[:,0:6], hue='class of diagnosis')
sns.pairplot(mesdata, y_vars=X_num.iloc[:,6:12], x_vars=X_num.iloc[:,6:12], hue='class of diagnosis')
sns.pairplot(mesdata, y_vars=X_num.iloc[:,12:], x_vars=X_num.iloc[:,12:], hue='class of diagnosis')
sns.pairplot(mesdata, y_vars=['class of diagnosis'], x_vars=X_num.iloc[:,0:6], hue='class of diagnosis')
sns.pairplot(mesdata, y_vars=['class of diagnosis'], x_vars=X_num.iloc[:,6:12], hue='class of diagnosis')
sns.pairplot(mesdata, y_vars=['class of diagnosis'], x_vars=X_num.iloc[:,12:], hue='class of diagnosis')

bp=sns.barplot(x=X['class of diagnosis'], y=X_num.iloc[:,1], hue='class of diagnosis', data=mesdata)

print(np.where((mesdata['platelet count (PLT)']>3000)))
print(np.where((mesdata['alkaline phosphatise (ALP)']>400)))
print(np.where((mesdata['pleural lactic dehydrogenise']>7000)))
print(np.where((mesdata['total protein']<4)))
print(np.where((mesdata['albumin']>6)))
# PLT affects about 3% recall across models
mesdata.drop(241, inplace = True)
# ALP dropped 19% recall LR, increase 0.08 KNN, dropped 0.05 SVM, dropped 0.18 NB, increase 0.02 CART, dropped 0.10 GB
mesdata.drop(29, inplace = True)
mesdata.drop(1, inplace = True)
mesdata.drop(251, inplace = True)
# Dropped 0.36 NB for all outliers dropped

from yellowbrick.regressor import CooksDistance

# Instantiate and fit the visualizer
visualizer = CooksDistance()
visualizer.fit(X, y)
visualizer.show()

# Boxplot
f, axes = plt.subplots(1,5)
sns.boxplot(y=X_num.iloc[:,0], x='class of diagnosis', data=X_num, orient='v', ax=axes[0])
sns.boxplot(y=X_num.iloc[:,1], x='class of diagnosis', data=X_num, orient='v', ax=axes[1])
sns.boxplot(y=X_num.iloc[:,2], x='class of diagnosis', data=X_num, orient='v', ax=axes[2])
sns.boxplot(y=X_num.iloc[:,3], x='class of diagnosis', data=X_num, orient='v', ax=axes[3])
sns.boxplot(y=X_num.iloc[:,4], x='class of diagnosis', data=X_num, orient='v', ax=axes[4])

f, axes = plt.subplots(1,5)
sns.boxplot(y=X_num.iloc[:,5], x='class of diagnosis', data=X_num, orient='v', ax=axes[0])
sns.boxplot(y=X_num.iloc[:,6], x='class of diagnosis', data=X_num, orient='v', ax=axes[1])
sns.boxplot(y=X_num.iloc[:,7], x='class of diagnosis', data=X_num, orient='v', ax=axes[2])
sns.boxplot(y=X_num.iloc[:,8], x='class of diagnosis', data=X_num, orient='v', ax=axes[3])
sns.boxplot(y=X_num.iloc[:,9], x='class of diagnosis', data=X_num, orient='v', ax=axes[4])

f, axes = plt.subplots(1,5)
sns.boxplot(y=X_num.iloc[:,10], x='class of diagnosis', data=X_num, orient='v', ax=axes[0])
sns.boxplot(y=X_num.iloc[:,11], x='class of diagnosis', data=X_num, orient='v', ax=axes[1])
sns.boxplot(y=X_num.iloc[:,12], x='class of diagnosis', data=X_num, orient='v', ax=axes[2])
sns.boxplot(y=X_num.iloc[:,13], x='class of diagnosis', data=X_num, orient='v', ax=axes[3])
sns.boxplot(y=X_num.iloc[:,14], x='class of diagnosis', data=X_num, orient='v', ax=axes[4])

f, axes = plt.subplots(1,2)
sns.boxplot(y=X_num.iloc[:,15], x='class of diagnosis', data=X_num, orient='v', ax=axes[0])
sns.boxplot(y=X_num.iloc[:,16], x='class of diagnosis', data=X_num, orient='v', ax=axes[1])

f, axes = plt.subplots(1,2)
sns.boxplot(y=mesdata['age'], x='class of diagnosis', data=mesdata, orient='v', ax=axes[0])
sns.boxplot(y=mesdata['C-reactive protein (CRP)'], x='class of diagnosis', data=mesdata, orient='v', ax=axes[1])