set.seed(42)

mesdata <- as.data.frame(read_excel
                         ("C:/Users/tedch/Documents/Thesis/Code/mesdata.xls"))
names(mesdata)[names(mesdata) == "class of diagnosis"] <- "diagnosis"
diagnosis <- mesdata$diagnosis
mesdata[,1:34] <- lapply(mesdata[,1:34], as.numeric)
transform<-c('duration of symptoms', 'platelet count (PLT)', 'alkaline phosphatise (ALP)', 'glucose',
             'pleural lactic dehydrogenise', 'cell count (WBC)', 'blood lactic dehydrogenise (LDH)')
mesdata[transform]<-lapply(mesdata[transform], log)
mesdata <- data.frame(mesdata)
mesdata$diagnosis <- as.factor(mesdata$diagnosis)
X <- subset(mesdata, select=-c(5, 7, 27, 30))


X <- X[sample(nrow(X)),]
X$diagnosis <- unclass(X$diagnosis)
data.train <- X[1:259,]
data.tst <- X[260:nrow(X),]

preproc <- preProcess(data.train, method = c("center", "scale"))
data.train <- predict(preproc, data.train[,])
data.tst <- predict(preproc, data.tst[,])

data.train$diagnosis <- as.factor(data.train$diagnosis)
data.train <- SMOTE(diagnosis ~ ., data.train, perc.over = 100)
data.train$diagnosis <- as.numeric(data.train$diagnosis)

control <- list(type.LU = "implicator.tnorm", k = 1, 
                type.aggregation = c("t.tnorm", "lukasiewicz"), 
                type.relation = c("tolerance", "eq.1"), t.implicator = "lukasiewicz") 	

kfold <- 8
cv <-createFolds(data.train$diagnosis,
                 k=kfold, list=TRUE, returnTrain=FALSE)

ptm <- proc.time()
for (i in 1:kfold){
  testIndex <- cv[[i]]
  
  trainingdata <- data.train[-testIndex,]
  traininglabel <- data.train[-testIndex,31]
  
  testingdata <- data.train[testIndex,1:30]
  testinglabel <- data.train[testIndex,31]
  
  real.X <- matrix(X[1:nrow(testingdata),31], ncol=1)
  decision.table <- SF.asDecisionTable(dataset = trainingdata, decision.attr = 31, 
  )
  tst.data <- SF.asDecisionTable(dataset = testingdata)
  res.test <- C.FRNN.FRST(decision.table = decision.table, newdata = tst.data,
                          control = control)
  pred <- data.frame(res.test)
  pred<-as.factor(pred$res.test)
  real <- data.frame(real.X)
  real<-as.factor(real$real.X)
}
print((proc.time() - ptm) /8)

confusionMatrix(pred,real, mode='everything', positive="2")
tmp <- prediction(res.test,real.X)
roc <- performance(tmp, "tpr", "fpr")
plot(roc, col = "blue", lwd = 2)
abline(a = 0, b = 1) 
auc <- performance(tmp, "auc")
auc <- as.numeric(auc@y.values)