set.seed(42)

mesdata <- as.data.frame(read_excel
                         ("C:/Users/tedch/Documents/Thesis/Code/mesdata.xls"))
names(mesdata)[names(mesdata) == "class of diagnosis"] <- "diagnosis"
diagnosis <- mesdata$diagnosis
mesdata[,1:34] <- lapply(mesdata[,1:34], as.numeric)
numcol<-c('age','duration of asbestos exposure', 'duration of symptoms', 'white blood', 'cell count (WBC)',
           'platelet count (PLT)', 'sedimentation', 'blood lactic dehydrogenise (LDH)', 'alkaline phosphatise (ALP)',
           'total protein', 'albumin', 'glucose', 'pleural lactic dehydrogenise', 'pleural protein', 'pleural albumin',
           'pleural glucose', 'C-reactive protein (CRP)')
mesdata <- mesdata[,numcol]
# XS = apply(mesdata, 2, function(x) (x-min(x))/(max(x) - min(x)))
transform<-c('duration of symptoms', 'platelet count (PLT)', 'alkaline phosphatise (ALP)', 'glucose',
             'pleural lactic dehydrogenise', 'cell count (WBC)', 'blood lactic dehydrogenise (LDH)')
mesdata[transform]<-lapply(mesdata[transform], log)
#mesdatascale<-scale(mesdata[,1:17])
X<-cbind(mesdata,diagnosis)
X <- data.frame(X)
# X$diagnosis <- as.factor(XS$diagnosis)
#X <- subset(mesdata, select=-c(7, 27))

X <- X[sample(nrow(X)),]
X$diagnosis <- unclass(X$diagnosis)
data.train <- X[1:259,]
data.tst <- X[260:nrow(X),]

preproc <- preProcess(data.train, method = c("range"), rangeBounds = c(0, 1))
data.train <- predict(preproc, data.train[,])
data.tst <- predict(preproc, data.tst[,])

#range.data <- apply(X[, -ncol(X)], 2, range)
#range.data <- matrix(c(0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
#                      0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
#                      0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1), nrow=2)
range.data <- matrix(c(0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
                       0,1,0,1,0,1,0,1,0,1), nrow=2)

data.train$diagnosis <- as.factor(data.train$diagnosis)
data.train <- SMOTE(diagnosis ~ ., data.train, perc.over = 100)
data.train$diagnosis <- as.numeric(data.train$diagnosis)

control <- list(num.class = 2, num.labels = 3, persen_cross = 0.3, max.iter = 10, max.gen = 10, 
                persen_mutant = 0.3, k.lower = 0, k.upper = 1, epsilon = 0.1) 

kfold <- 8
cv <-createFolds(data.train$diagnosis,
                 k=kfold, list=TRUE, returnTrain=FALSE)

ptm <- proc.time()
for (i in 1:kfold ){
  testIndex <- cv[[i]]
  
  trainingdata <- data.train[-testIndex,]
  traininglabel <- data.train[-testIndex,18]
  
  testingdata <- data.train[testIndex,1:17]
  testinglabel <- data.train[testIndex,18]
  
  real.X <- matrix(X[1:nrow(testingdata),18], ncol=1)
  object <- frbs.learn(trainingdata, range.data, method.type=c("SLAVE"), control)
  res.test <- predict(object, testingdata)
  pred <- data.frame(res.test)
  pred<-as.factor(pred$res.test)
  real <- data.frame(real.X)
  real<-as.factor(real$real.X)
}
print((proc.time() - ptm) /8)

confusionMatrix(pred,real, positive="2")
tmp <- prediction(res.test,real.X)
roc <- performance(tmp, "tpr", "fpr")
plot(roc, col = "blue", lwd = 2)
abline(a = 0, b = 1) 
auc <- performance(tmp, "auc")
auc <- as.numeric(auc@y.values)
